from django.conf import settings
from django.db import models
from django.utils import timezone


class Device(models.Model):
    user = models.ForeignKey(
        to=settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name="devices",
    )

    lang = models.CharField(
        # e.g. for ISO 639-1 & ISO 3166-1 alpha-2
        max_length=5,
        blank=True,
    )

    push_token = models.CharField(
        # https://github.com/expo/expo/issues/1135#issuecomment-399622890
        max_length=4096,
    )

    date_registered = models.DateTimeField(
        default=timezone.now,
    )

    is_active = models.BooleanField(
        default=True,
    )

    class Meta:
        unique_together = (
            "user",
            "push_token",
        )

    def __str__(self) -> str:
        return f"Device #{self.pk} of {self.user.username}"



from datetime import timedelta

from django.conf import settings as django_settings


class Settings:
    @property
    def token(self) -> str | None:
        return getattr(
            django_settings,
            "EXPO_NOTIFICATIONS_TOKEN",
            None,
        )

    @property
    def receipt_check_delay(self) -> timedelta:
        return getattr(
            django_settings,
            "EXPO_NOTIFICATIONS_RECEIPT_CHECK_DELAY",
            timedelta(minutes=30),
        )

    @property
    def sending_task_max_retries(self) -> int:
        return getattr(
            django_settings,
            "EXPO_NOTIFICATIONS_SENDING_TASK_MAX_RETRIES",
            5,
        )

    @property
    def sending_task_retry_delay(self) -> timedelta:
        return getattr(
            django_settings,
            "EXPO_NOTIFICATIONS_SENDING_TASK_RETRY_DELAY",
            timedelta(seconds=30),
        )

    @property
    def checking_task_max_retries(self) -> int:
        return getattr(
            django_settings,
            "EXPO_NOTIFICATIONS_CHECKING_TASK_MAX_RETRIES",
            3,
        )

    @property
    def checking_task_retry_delay(self) -> timedelta:
        return getattr(
            django_settings,
            "EXPO_NOTIFICATIONS_CHECKING_TASK_RETRY_DELAY",
            timedelta(minutes=1),
        )


settings = Settings()



from django.apps import AppConfig


class ExpoNotificationsConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "expo_notifications"
    verbose_name = "Expo Notifications"



import requests

from expo_notifications.conf import settings

session = requests.Session()
session.headers.update(
    {
        "accept": "application/json",
        "accept-encoding": "gzip, deflate",
        "content-type": "application/json",
    }
)

if settings.token is not None:
    session.headers.update({"Authorization": f"Bearer {settings.token}"})



from celery import shared_task
from django.utils import timezone
from exponent_server_sdk import (
    DeviceNotRegisteredError,
    PushClient,
    PushReceipt,
    PushServerError,
    PushTicketError,
)
from requests.exceptions import ConnectionError, HTTPError

from expo_notifications.conf import settings
from expo_notifications.models import Receipt, Ticket
from expo_notifications.tasks.session import session


@shared_task(
    bind=True,
    ignore_result=True,
    max_retries=settings.checking_task_max_retries,
    default_retry_delay=settings.checking_task_retry_delay.total_seconds(),
)
def check_receipts(self, ticket_pks: list[str]) -> None:
    tickets = Ticket.objects.filter(pk__in=ticket_pks).exclude(external_id="")

    push_tickets = [ticket.to_push_ticket() for ticket in tickets]

    push_client = PushClient(session=session)

    try:
        push_receipts: list[PushReceipt] = push_client.check_receipts_multiple(
            push_tickets
        )
    except PushServerError:
        raise self.retry()
    except (ConnectionError, HTTPError):
        raise self.retry()

    receipts: list[Receipt] = []

    for push_receipt in push_receipts:
        ticket = tickets.get(external_id=push_receipt.id)

        try:
            push_receipt.validate_response()
        except DeviceNotRegisteredError:
            ticket.message.device.is_active = False
            ticket.message.device.save()
        except PushTicketError:
            pass

        receipts.append(
            Receipt(
                ticket=ticket,
                is_success=push_receipt.is_success(),
                error_message=push_receipt.message,
                date_checked=timezone.now(),
            )
        )

    Receipt.objects.bulk_create(receipts)
